import re
import os
import datetime
import tempfile
import subprocess


def filter_to_string(filters, search_query):
    pieces = []
    category = filters.get("category")
    status = filters.get("status")
    if category:
        pieces += [f" category:{category}"]
    if status:
        pieces += [f" status:{status}"]
    if search_query:
        pieces += [f" search:'{search_query}"]
    return "".join(pieces)


def remove_rich_tag(text):
    """remove rich styling from a string"""
    pattern = r"\[[^\]]*\](.*?)\[/\]"
    return re.sub(pattern, r"\1", text)


def advance_enum_val(enum_type, cur_val):
    """advance a value in an enum by one, wrapping around"""
    members = [str(e.value) for e in enum_type]
    cur_idx = members.index(remove_rich_tag(cur_val))
    next_idx = (cur_idx + 1) % len(members)
    return members[next_idx]


def get_color_enum(value: str, enum: dict[str, dict], default: str) -> str:
    color = enum.get(value, {"color": default})["color"]
    return f"[{color}]{value}[/]"


def get_colored_category(category: str) -> str:
    hash_val = sum(ord(c) for c in category)
    hue = hash_val % 360
    color = f"rgb({hue},200,200) on default"
    return f"[{color}]{category}[/]"


def get_colored_date(date: datetime.date) -> str:
    if not isinstance(date, datetime.date):
        return ""
    as_str = date.strftime("%Y-%m-%d")
    today = datetime.date.today()
    if date.date() < today:
        return f"[#eeeeee on #dd1111]{as_str}[/]"
    else:
        # Calculate weeks into future
        delta = date.date() - today
        weeks = delta.days // 7

        colors = [
            # "#FF4000",
            "#FF8000",
            "#FFA533",
            "#FFD24D",
            "#FFE680",
            "#FFF4B3",
            "#FFFFFF",
        ]

        if weeks >= len(colors):
            color_index = -1
        else:
            color_index = weeks
        color = colors[color_index]
    return f"[{color}]{as_str}[/]"


def get_text_from_editor(initial_text: str = "") -> str | None:
    editor = os.environ.get("EDITOR", "vim")

    with tempfile.NamedTemporaryFile(suffix=".txt", mode="w+", delete=True) as tf:
        tf.write(initial_text)
        tf.flush()

        initial_mtime = os.path.getmtime(tf.name)
        subprocess.call([editor, tf.name])

        if os.path.getmtime(tf.name) == initial_mtime:
            return None

        with open(tf.name, "r") as f:  # Open fresh to get new content
            edited_text = f.read()
            return edited_text
